CKEDITOR.plugins.add('letterspacing', {
    requires: ['richcombo'],
    lang: 'en,zh-cn',
    init: function (editor) {
        let styles_tmpl = {
            element: 'span',
            styles: { 'letter-spacing': '#(size)' },
            overrides: [ {
                element: 'letter-spacing', attributes: { 'size': null }
            } ]
        };

        var config = editor.config,
            lang = editor.lang.format;
        var trackings = ["1px", "1.5px", "2px", "2.5px", "3px", "4px", "5px", "6px", "7px", "8px", "10px", "15px"];

        var styles = {},values = [];
        for ( var i = 0; i < trackings.length; i++ ) {
            var parts = trackings[ i ];
            if ( parts ) {
                parts = parts.split( '/' );
                var vars = {},name = trackings[ i ] = parts[ 0 ];
                vars['size'] = values[ i ] = parts[ 1 ] || name;
                styles[ name ] = new CKEDITOR.style( styles_tmpl, vars );
                styles[ name ]._.definition.name = name;
            }
        }

        config.allowedContent = 'span'; //There may be a better way to do this.

        editor.ui.addRichCombo('letterspacing', {
            label: editor.lang.letterspacing.title,
            title: 'Change letter-spacing',
            voiceLabel: 'Change letter-spacing',
            className: 'cke_letterspacing',
            multiSelect: false,

            panel: {
                css: [config.contentsCss, CKEDITOR.getUrl(CKEDITOR.skin.getPath('editor') + 'editor.css')]
            },

            init: function () {
                this.startGroup(editor.lang.letterspacing.title);
                for (var this_letting in trackings) {
                    this.add(trackings[this_letting], trackings[this_letting], trackings[this_letting]);
                    if (this_letting == "11") {
                        return false;
                    }
                }
            },

            onClick: function (value,aaa,bbb) {
                editor.focus();
                editor.fire('saveSnapshot');
                var ep = editor.elementPath();
                var style = new CKEDITOR.style({ styles: { 'letter-spacing': value } });
                editor[style.checkActive(ep) ? 'removeStyle' : 'applyStyle'](style);
                let label = editor.container.$.getElementsByClassName("cke_letterspacing");
                label = label[0].getElementsByClassName("cke_combo_inlinelabel")[0];
                if(label)label.innerHTML = value;
                editor.fire('saveSnapshot');
            },

            onRender: function() {
                editor.on( 'selectionChange', function( ev ) {
                    var currentValue = this.getValue();
                    var elementPath = ev.data.path,elements = elementPath.elements;
                    for ( var i = 0, element; i < elements.length; i++ ) {
                        element = elements[ i ];
                        for ( var value in styles ) {
                            if ( styles[ value ].checkElementMatch( element, true, editor ) ) {
                                if ( value != currentValue )
                                    this.setValue( value );
                                return;
                            }
                        }
                    }
                    this.setValue( '', editor.lang.letterspacing.title );
                }, this );
            }

        });
    }
});
