<?php
/**
 *
 * example目录下为简单的支付样例，仅能用于搭建快速体验微信支付使用
 * 样例的作用仅限于指导如何使用sdk，在安全上面仅做了简单处理， 复制使用样例代码时请慎重
 * 请勿直接直接使用样例对外提供服务
 *
 **/

require_once "AliPay.Config.php";
require_once "AopClient.php";
require_once APP_PATH . "api/service/PayBusinessCallBack.php";

use think\Log;
use think\Db;
use app\api\service\PayBusinessCallBack;

class PayNotifyCallBack
{
    public function Handle($objData)
    {
        $this->NotifyProcess($objData);
    }

    //重写回调处理函数
    public function NotifyProcess($objData)
    {
        Log::write('支付宝支付回调参数：' . json_encode($objData));
        if (!empty($objData)) {
            if (isset($objData['passback_params'])) {
                $passback_params = json_decode(urlencode($objData['passback_params']), true);
            }

            $out_trade_no = $objData['out_trade_no'];

            // 查询订单信息
            $order_info = Db::name("sys_order")->field('id,user_id')->where(["orderno" => $out_trade_no])->find();
            if (!$order_info) {
                echo "fail";
                return;
            }

            // 查询用户信息
            $user_info = Db::name("users")->field('agent')->where(["uid" => $order_info['user_id']])->find();
            if (!$user_info) {
                echo "fail";
                return;
            }

            // 查询支付配置用，平台配置还是代理商配置
            $pay_setting_key = 'pay_setting';
            if ($user_info['agent'] > 0) {
                // 查询所属代理商id
                $user_info = Db::name("users")->field('uid')->where(["uid" => $user_info['agent']])->find();
                $pay_setting_key = 'pay_setting_'.$user_info['uid'];
            }

            $config = new \AliPayConfig($pay_setting_key);
            $aopClient = new \AopClient();
            $aopClient->appId = $config->GetAppId();
            $aopClient->rsaPrivateKey = $config->GetPrivateKey();
            $aopClient->alipayrsaPublicKey = $config->GetPublicKey();

            if (!$aopClient->rsaCheckV1($objData, '', 'RSA2')) {
                Log::write("支付宝签名错误...");
                echo "fail";
                return;
            }

            // 记录回调结果，便于调试
            $pay_notify['order_number'] = $out_trade_no;
            $pay_notify['notify_message'] = json_encode($objData);
            $pay_notify['create_time'] = time();
            Db::name('sys_pay_notify')->insert($pay_notify);

            $payBusinessCallBack = new PayBusinessCallBack();
            $payBusinessCallBack->checkOrder($out_trade_no, $objData['trade_no']);

            echo "success";
            return;
        } else {
            echo "fail";
            return;
        }
    }
}

