<?php
/**
 * 支付统一入口
 * Created by PhpStorm.
 * User: Administrator
 * Date: 2019/9/24
 * Time: 19:41
 */

require_once IA_ROOT . "/framework/Wxpay/lib/WxPay.Api.php";
require_once IA_ROOT . "/framework/AliSDK/AliPay.Config.php";
require_once IA_ROOT . "/framework/AliSDK/AopClient.php";
require_once IA_ROOT . "/framework/AliSDK/request/AlipayTradePagePayRequest.php";
require_once IA_ROOT . "/framework/QrCode/QRcode.php";

use think\Db;

class Pay
{
    private $error_msg_arr = [
        'INVALID_REQUEST' => '支付参数错误',
        'NOAUTH' => '商户无此接口权限',
        'NOTENOUGH' => '余额不足',
        'ORDERPAID' => '订单已支付，请联系商家',
        'ORDERCLOSED' => '订单已关闭，请重新下单',
        'SYSTEMERROR' => '系统异常，请稍后重试',
        'APPID_NOT_EXIST' => '支付参数APPID不存在',
        'MCHID_NOT_EXIST' => '支付参数MCHID不存在',
        'APPID_MCHID_NOT_MATCH' => '微信支付商户号和小程序APPID未关联',
        'LACK_PARAMS' => '系统异常，缺少必要的请求参数',
        'OUT_TRADE_NO_USED' => '系统异常，商户订单号重复',
        'SIGNERROR' => '签名错误，请重置支付密钥',
        'XML_FORMAT_ERROR' => '系统异常，XML格式错误',
        'REQUIRE_POST_METHOD' => '系统异常，请使用post方法',
        'POST_DATA_EMPTY' => '系统异常，post数据为空',
        'NOT_UTF8' => '系统异常，编码格式错误',
    ];

    /**
     * 微信支付
     * @param string $body
     * @param $pay_type 1/小程序支付 2/公众号 3/H5支付
     * @return array
     * @throws WxPayException
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function Wxpay($data = [], $pay_type = 1, $return_url = '')
    {
        $rs = array('code' => 200, 'info' => array(), 'msg' => '');
        if (empty($data)) {
            $rs['code'] = 0;
            $rs['msg'] = '订单不存在';
            return $rs;
        }
        //初始化值对象
        $input = new \WxPayUnifiedOrder();
        $input->SetBody(empty($data['body']) ? '微信支付' : $data['body']);
        $input->SetDetail(empty($data['detail']) ? '微信支付' : $data['detail']);
        $input->SetTotal_fee($data['pay_money'] * 100);
        $input->SetOut_trade_no($data['out_trade_no']);

        if ($pay_type == 1 || $pay_type == 2) {
            $input->SetOpenid($data['openid']);
            $input->SetTrade_type("JSAPI");
        } elseif ($pay_type == 3) {
            $input->SetTrade_type("MWEB");
            $input->SetScene_info('{"h5_info": {"type":"Wap","wap_url": "","wap_name": ""}}');
        } elseif ($pay_type == 4) {
            $input->SetTrade_type("APP");
        } elseif ($pay_type == 5) {
            $input->SetProduct_id($data['out_trade_no']);
            $input->SetTrade_type("NATIVE");
        }
        if (!empty($data['attach'])) {
            $input->SetAttach($data['attach']);
        }
        // 向微信统一下单，并返回 unifiedorder，它是一个array数组1
        $unifiedorder = \WxPayApi::unifiedOrder($input);
        //return $unifiedorder;
        //下单错误
        if ($unifiedorder['return_code'] == "FAIL") {
            $rs['code'] = 0;
            $rs['msg'] = $unifiedorder['return_msg'] == '签名错误' ? '签名错误，请重置支付密钥' : $unifiedorder['return_msg'];
            return $rs;
        }
        if ($unifiedorder['result_code'] == "FAIL") {
            $rs['code'] = 0;
            $rs['msg'] = isset($this->error_msg_arr[$unifiedorder['err_code']]) ? $this->error_msg_arr[$unifiedorder['err_code']] : $unifiedorder['err_code_des'];
            return $rs;
        }
        if ($pay_type == 1 || $pay_type == 2) {
            $parameters = \WxPayApi::getJsApiParameters($unifiedorder);
        } elseif ($pay_type == 3) {
            if (!empty($return_url)) {
                $unifiedorder['mweb_url'] = $unifiedorder['mweb_url'] . '&redirect_url=' . urlencode($return_url);
            }
            $parameters = $unifiedorder;
        } elseif ($pay_type == 4) {
            $parameters = \WxPayApi::getAppParameters($unifiedorder);
        } elseif ($pay_type == 5) {
            $qrcode = new \QrCode\QRcode();
            $parameters = $qrcode::dataImgBase64($unifiedorder['code_url'], QR_ECLEVEL_L, 8);
        }
        $rs['info'] = $parameters;
        return $rs;
    }

    public function AliWebPay($data = [], $return_url = '')
    {
        $rs = array('code' => 200, 'info' => array(), 'msg' => '');
        /*支付宝支付*/
        $aliConfig = new \AliPayConfig();
        $aopClient = new \AopClient();
        $aopClient->appId = $aliConfig->GetAppId();
        $aopClient->rsaPrivateKey = $aliConfig->GetPrivateKey();
        $aopClient->alipayrsaPublicKey = $aliConfig->GetPublicKey();
        $aopClient->signType = $aliConfig->GetSignType();
        $request = new \AlipayTradePagePayRequest();
        $request->setNotifyUrl($aliConfig->GetNotifyUrl());
        $request->setReturnUrl($return_url);
        $biz_content = array(
            'subject' => empty($data['body']) ? '支付宝支付' : $data['body'],
            'out_trade_no' => $data['out_trade_no'],
            'total_amount' => $data['pay_money'],
            'product_code' => 'FAST_INSTANT_TRADE_PAY',
        );
        if (!empty($data['attach'])) {
            $biz_content['passback_params'] = urlencode($data['attach']);
        }
        $request->setBizContent(json_encode($biz_content));
        //$response = $aopClient->pageExecute($request,'GET');
        $response = $aopClient->pageExecute($request,'GET');
        $rs['info'] = $response;
        return $rs;
    }
}