<?php
defined('IN_IA') or exit('Access Denied');
define('TIMESTAMP', time());
require_once IA_ROOT . 'framework/Curl.php';

class WeixinAccount
{
    protected $account;
    public $uniacid;

    public function __construct($uniaccount = array(), $user_id = 0)
    {
        if (empty($uniaccount)) {
            $uniaccount = \think\Db::name('sys_account_wechats')->where('user_id', $user_id)->find();
        }
        $this->uniacid = $uniaccount['uniacid'];
        $this->account = $uniaccount;
        $this->account['encrypt_key'] = $uniaccount['key'];
    }

    public function checkSign()
    {
        $token = $this->account['token'];
        $signkey = array($token, $_GET['timestamp'], $_GET['nonce']);
        sort($signkey, SORT_STRING);
        $signString = implode($signkey);
        $signString = sha1($signString);
        return $signString == $_GET['signature'];
    }

    public function checkSignature($encrypt_msg)
    {
        $str = $this->buildSignature($encrypt_msg);
        return $str == $_GET['msg_signature'];
    }


    public function buildSignature($encrypt_msg)
    {
        $token = $this->account['token'];
        $array = array($encrypt_msg, $token, $_GET['timestamp'], $_GET['nonce']);
        sort($array, SORT_STRING);
        $str = implode($array);
        $str = sha1($str);
        return $str;
    }


    public function encryptMsg($text)
    {
        $token = $this->account['token'];
        $encodingaeskey = $this->account['encodingaeskey'];
        $appid = $this->account['encrypt_key'];

        $key = base64_decode($encodingaeskey . '=');
        $text = random(16) . pack("N", strlen($text)) . $text . $appid;
        $iv = substr($key, 0, 16);
        $block_size = 32;
        $text_length = strlen($text);
        $amount_to_pad = $block_size - ($text_length % $block_size);
        if ($amount_to_pad == 0) {
            $amount_to_pad = $block_size;
        }
        $pad_chr = chr($amount_to_pad);
        $tmp = '';
        for ($index = 0; $index < $amount_to_pad; $index++) {
            $tmp .= $pad_chr;
        }
        $text = $text . $tmp;

        $encrypted = openssl_encrypt($text, 'AES-256-CBC', $key, OPENSSL_RAW_DATA | OPENSSL_ZERO_PADDING, $iv);
        $encrypt_msg = base64_encode($encrypted);
        $signature = $this->buildSignature($encrypt_msg);
        return array($signature, $encrypt_msg);
    }


    public function decryptMsg($postData)
    {
        $token = $this->account['token'];
        $encodingaeskey = $this->account['encodingaeskey'];
        $appid = $this->account['encrypt_key'];
        $key = base64_decode($encodingaeskey . '=');

        if (strlen($encodingaeskey) != 43) {
            return error(-1, "微信公众平台返回接口错误. \n错误代码为: 40004 \n,错误描述为: " . $this->encryptErrorCode('40004'));
        }
        $packet = $this->xmlExtract($postData);
        if (is_error($packet)) {
            return error(-1, $packet['message']);
        }
        $istrue = $this->checkSignature($packet['encrypt']);
        if (!$istrue) {
            return error(-1, "微信公众平台返回接口错误. \n错误代码为: 40001 \n,错误描述为: " . $this->encryptErrorCode('40001'));
        }
        $ciphertext_dec = base64_decode($packet['encrypt']);
        $iv = substr($key, 0, 16);
        $decrypted = openssl_decrypt($ciphertext_dec, 'AES-256-CBC', $key, OPENSSL_RAW_DATA | OPENSSL_ZERO_PADDING, $iv);

        $pad = ord(substr($decrypted, -1));
        if ($pad < 1 || $pad > 32) {
            $pad = 0;
        }
        $result = substr($decrypted, 0, (strlen($decrypted) - $pad));
        if (strlen($result) < 16) {
            return '';
        }
        $content = substr($result, 16, strlen($result));
        $len_list = unpack("N", substr($content, 0, 4));
        $xml_len = $len_list[1];
        $xml_content = substr($content, 4, $xml_len);
        $from_appid = substr($content, $xml_len + 4);
        if ($from_appid != $appid) {
            return error(-1, "微信公众平台返回接口错误. \n错误代码为: 40005 \n,错误描述为: " . $this->encryptErrorCode('40005'));
        }
        return $xml_content;
    }


    function xmlDetract($data)
    {
        $xml['Encrypt'] = $data[1];
        $xml['MsgSignature'] = $data[0];
        $xml['TimeStamp'] = $_GET['timestamp'];
        $xml['Nonce'] = $_GET['nonce'];
        return array2xml($xml);
    }


    public function xmlExtract($message)
    {
        $packet = array();
        if (!empty($message)) {
            $obj = isimplexml_load_string($message, 'SimpleXMLElement', LIBXML_NOCDATA);
            if ($obj instanceof SimpleXMLElement) {
                $packet['encrypt'] = strval($obj->Encrypt);
                $packet['to'] = strval($obj->ToUserName);
            }
        }
        if (!empty($packet['encrypt'])) {
            return $packet;
        } else {
            return error(-1, "微信公众平台返回接口错误. \n错误代码为: 40002 \n,错误描述为: " . $this->encryptErrorCode('40002'));
        }
    }

    public function response($packet)
    {
        if (is_error($packet)) {
            return '';
        }
        if (!is_array($packet)) {
            return $packet;
        }
        if (empty($packet['CreateTime'])) {
            $packet['CreateTime'] = TIMESTAMP;
        }
        if (empty($packet['MsgType'])) {
            $packet['MsgType'] = 'text';
        }
        if (empty($packet['FuncFlag'])) {
            $packet['FuncFlag'] = 0;
        } else {
            $packet['FuncFlag'] = 1;
        }
        return array2xml($packet);
    }

    public function parse($message)
    {
        if (!empty($message)) {
            $message = xml2array($message);
            $packet = iarray_change_key_case($message, CASE_LOWER);
            $packet['from'] = $message['FromUserName'];
            $packet['to'] = $message['ToUserName'];
            $packet['time'] = $message['CreateTime'];
            $packet['type'] = $message['MsgType'];
            $packet['msgtype'] = $message['MsgType'];
            $packet['event'] = isset($message['Event']) ? $message['Event'] : '';
            switch ($packet['type']) {
                case 'text':
                    $packet['redirection'] = false;
                    $packet['source'] = null;
                    break;
                case 'image':
                    $packet['url'] = $message['PicUrl'];
                    break;
                case 'video':
                case 'shortvideo':
                    $packet['thumb'] = $message['ThumbMediaId'];
                    break;
            }

            switch ($packet['event']) {
                case 'subscribe':
                    $packet['type'] = 'subscribe';
                case 'SCAN':
                    if ($packet['event'] == 'SCAN') {
                        $packet['type'] = 'qr';
                    }
                    if (!empty($packet['eventkey'])) {
                        $packet['scene'] = str_replace('qrscene_', '', $packet['eventkey']);
                        if (strexists($packet['scene'], '\u')) {
                            $packet['scene'] = '"' . str_replace('\\u', '\u', $packet['scene']) . '"';
                            $packet['scene'] = json_decode($packet['scene']);
                        }

                    }
                    break;
                case 'unsubscribe':
                    $packet['type'] = 'unsubscribe';
                    break;
                case 'LOCATION':
                    $packet['type'] = 'trace';
                    $packet['location_x'] = $message['Latitude'];
                    $packet['location_y'] = $message['Longitude'];
                    break;
                case 'pic_photo_or_album':
                case 'pic_weixin':
                case 'pic_sysphoto':
                    $packet['sendpicsinfo']['piclist'] = array();
                    $packet['sendpicsinfo']['count'] = $message['SendPicsInfo']['Count'];
                    if (!empty($message['SendPicsInfo']['PicList'])) {
                        foreach ($message['SendPicsInfo']['PicList']['item'] as $item) {
                            if (empty($item)) {
                                continue;
                            }
                            $packet['sendpicsinfo']['piclist'][] = is_array($item) ? $item['PicMd5Sum'] : $item;
                        }
                    }
                    break;
                case 'card_pass_check':
                case 'card_not_pass_check':
                case 'user_get_card':
                case 'user_del_card':
                case 'user_consume_card':
                case 'poi_check_notify':
                    $packet['type'] = 'coupon';
                    break;
            }
        }
        return $packet;
    }

    public function errorCode($code, $errmsg = '未知错误')
    {
        $errors = array(
            '-1' => '系统繁忙',
            '0' => '请求成功',
            '40001' => '获取access_token时AppSecret错误，或者access_token无效',
            '40002' => '不合法的凭证类型',
            '40003' => '不合法的OpenID',
            '40004' => '不合法的媒体文件类型',
            '40005' => '不合法的文件类型',
            '40006' => '不合法的文件大小',
            '40007' => '不合法的媒体文件id',
            '40008' => '不合法的消息类型',
            '40009' => '不合法的图片文件大小',
            '40010' => '不合法的语音文件大小',
            '40011' => '不合法的视频文件大小',
            '40012' => '不合法的缩略图文件大小',
            '40013' => '不合法的APPID',
            '40014' => '不合法的access_token',
            '40015' => '不合法的菜单类型',
            '40016' => '不合法的按钮个数',
            '40017' => '不合法的按钮个数',
            '40018' => '不合法的按钮名字长度',
            '40019' => '不合法的按钮KEY长度',
            '40020' => '不合法的按钮URL长度',
            '40021' => '不合法的菜单版本号',
            '40022' => '不合法的子菜单级数',
            '40023' => '不合法的子菜单按钮个数',
            '40024' => '不合法的子菜单按钮类型',
            '40025' => '不合法的子菜单按钮名字长度',
            '40026' => '不合法的子菜单按钮KEY长度',
            '40027' => '不合法的子菜单按钮URL长度',
            '40028' => '不合法的自定义菜单使用用户',
            '40029' => '不合法的oauth_code',
            '40030' => '不合法的refresh_token',
            '40031' => '不合法的openid列表',
            '40032' => '不合法的openid列表长度',
            '40033' => '不合法的请求字符，不能包含\uxxxx格式的字符',
            '40035' => '不合法的参数',
            '40038' => '不合法的请求格式',
            '40039' => '不合法的URL长度',
            '40050' => '不合法的分组id',
            '40051' => '分组名字不合法',
            '40155' => '请勿添加其他公众号的主页链接',
            '41001' => '缺少access_token参数',
            '41002' => '缺少appid参数',
            '41003' => '缺少refresh_token参数',
            '41004' => '缺少secret参数',
            '41005' => '缺少多媒体文件数据',
            '41006' => '缺少media_id参数',
            '41007' => '缺少子菜单数据',
            '41008' => '缺少oauth code',
            '41009' => '缺少openid',
            '42001' => 'access_token超时',
            '42002' => 'refresh_token超时',
            '42003' => 'oauth_code超时',
            '43001' => '需要GET请求',
            '43002' => '需要POST请求',
            '43003' => '需要HTTPS请求',
            '43004' => '需要接收者关注',
            '43005' => '需要好友关系',
            '44001' => '多媒体文件为空',
            '44002' => 'POST的数据包为空',
            '44003' => '图文消息内容为空',
            '44004' => '文本消息内容为空',
            '45001' => '多媒体文件大小超过限制',
            '45002' => '消息内容超过限制',
            '45003' => '标题字段超过限制',
            '45004' => '描述字段超过限制',
            '45005' => '链接字段超过限制',
            '45006' => '图片链接字段超过限制',
            '45007' => '语音播放时间超过限制',
            '45008' => '图文消息超过限制',
            '45009' => '接口调用超过限制',
            '45010' => '创建菜单个数超过限制',
            '45015' => '回复时间超过限制',
            '45016' => '系统分组，不允许修改',
            '45017' => '分组名字过长',
            '45018' => '分组数量超过上限',
            '45056' => '创建的标签数过多，请注意不能超过100个',
            '45057' => '该标签下粉丝数超过10w，不允许直接删除',
            '45058' => '不能修改0/1/2这三个系统默认保留的标签',
            '45059' => '有粉丝身上的标签数已经超过限制',
            '45065' => '24小时内不可给该组人群发该素材',
            '45157' => '标签名非法，请注意不能和其他标签重名',
            '45158' => '标签名长度超过30个字节',
            '45159' => '非法的标签',
            '46001' => '不存在媒体数据',
            '46002' => '不存在的菜单版本',
            '46003' => '不存在的菜单数据',
            '46004' => '不存在的用户',
            '47001' => '解析JSON/XML内容错误',
            '48001' => 'api功能未授权',
            '48003' => '请在微信平台开启群发功能',
            '50001' => '用户未授权该api',
            '40070' => '基本信息baseinfo中填写的库存信息SKU不合法。',
            '41011' => '必填字段不完整或不合法，参考相应接口。',
            '40056' => '无效code，请确认code长度在20个字符以内，且处于非异常状态（转赠、删除）。',
            '43009' => '无自定义SN权限，请参考开发者必读中的流程开通权限。',
            '43010' => '无储值权限,请参考开发者必读中的流程开通权限。',
            '43011' => '无积分权限,请参考开发者必读中的流程开通权限。',
            '40078' => '无效卡券，未通过审核，已被置为失效。',
            '40079' => '基本信息base_info中填写的date_info不合法或核销卡券未到生效时间。',
            '45021' => '文本字段超过长度限制，请参考相应字段说明。',
            '40080' => '卡券扩展信息cardext不合法。',
            '40097' => '基本信息base_info中填写的参数不合法。',
            '49004' => '签名错误。',
            '43012' => '无自定义cell跳转外链权限，请参考开发者必读中的申请流程开通权限。',
            '40099' => '该code已被核销。',
            '61005' => '缺少接入平台关键数据，等待微信开放平台推送数据，请十分钟后再试或是检查“授权事件接收URL”是否写错（index.php?c=account&amp;a=auth&amp;do=ticket地址中的&amp;符号容易被替换成&amp;amp;）',
            '61023' => '请重新授权接入该公众号',
            '88000' => '没有留言权限',
            '88001' => '该图文不存在',
            '88002' => '文章存在敏感信息',
            '88003' => '精选评论数已达上限',
            '88004' => '已被用户删除，无法精选',
            '88005' => '已经回复过了',
            '88007' => '回复超过长度限制或为0',
            '88008' => '该评论不存在',
            '88010' => '获取评论数目不合法',
            '87009' => '该回复不存在',
        );
        $code = strval($code);
        if ($code == '40001' || $code == '42001') {
            cache_delete(cache_system_key('accesstoken', array('uniacid' => $this->account['uniacid'])));
            return '微信公众平台授权异常, 系统已修复这个错误, 请刷新页面重试.';
        }

        if ($code == '40164') {
            $pattern = "((([0-9]{1,3})(\.)){3}([0-9]{1,3}))";
            preg_match($pattern, $errmsg, $out);

            $ip = !empty($out) ? $out[0] : '';
            return '获取微信公众号授权失败，错误代码:' . $code . ' 错误信息: ip-' . $ip . '不在白名单之内！';
        }

        if ($errors[$code]) {
            return $errors[$code];
        } else {
            return $errmsg;
        }
    }

    private function encryptErrorCode($code)
    {
        $errors = array(
            '40001' => '签名验证错误',
            '40002' => 'xml解析失败',
            '40003' => 'sha加密生成签名失败',
            '40004' => 'encodingAesKey 非法',
            '40005' => 'appid 校验错误',
            '40006' => 'aes 加密失败',
            '40007' => 'aes 解密失败',
            '40008' => '解密后得到的buffer非法',
            '40009' => 'base64加密失败',
            '40010' => 'base64解密失败',
            '40011' => '生成xml失败',
        );
        if ($errors[$code]) {
            return $errors[$code];
        } else {
            return '未知错误';
        }
    }

    public function getAccessToken()
    {
        $cache = get_sys_core_cache('accesstoken' . $this->account['key']);
        if (!empty($cache) && !empty($cache['token']) && $cache['expire'] > TIMESTAMP) {
            $this->account['access_token'] = $cache;
            return $cache['token'];
        }
        if (empty($this->account['key']) || empty($this->account['secret'])) {
            return error('-1', '未填写公众号的 appid 或 appsecret！');
        }
        $url = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid={$this->account['key']}&secret={$this->account['secret']}";
        $curl = new \Curl($url);
        $token = $curl->post(CURL_RETURN_JSON);
        if (is_error($token)) {
            return error('-1', '获取微信公众号授权失败, 请稍后重试！错误详情: ' . $token['message']);
        }
        if ($token['errcode'] == '40164') {
            return error(-1, $this->errorCode($token['errcode'], $token['errmsg']));
        }
        if (empty($token) || !is_array($token) || empty($token['access_token']) || empty($token['expires_in'])) {
            return error('-1', '获取微信公众号授权失败, 请稍后重试！ 公众平台返回原始数据为: 错误代码-' . $token['errcode'] . '，错误信息-' . $token['errmsg']);
        }
        $record = array();
        $record['token'] = $token['access_token'];
        $record['expire'] = TIMESTAMP + $token['expires_in'] - 200;
        $this->account['access_token'] = $record;
        upd_sys_core_cache('accesstoken' . $this->account['key'], $record);
        return $record['token'];
    }

    public function clearAccessToken()
    {
        $access_token = $this->getAccessToken();
        if (is_error($access_token)) {
            return $access_token;
        }
        $url = 'https://api.weixin.qq.com/cgi-bin/getcallbackip?access_token=' . $access_token;
        $curl = new \Curl($url);
        $response = $curl->post(CURL_RETURN_JSON);
        if (is_error($response) && $response['errno'] == '40001') {
        }
        return true;
    }

    public function getJsApiTicket(){
        $cache = get_sys_core_cache('jsticket' . $this->account['key']);
        if(!empty($cache) && !empty($cache['ticket']) && $cache['expire'] > TIMESTAMP) {
            return $cache['ticket'];
        }
        $access_token = $this->getAccessToken();
        if(is_error($access_token)){
            return $access_token;
        }
        $url = "https://api.weixin.qq.com/cgi-bin/ticket/getticket?access_token={$access_token}&type=jsapi";
        $curl = new \Curl($url);
        $content = $curl->post(CURL_RETURN_JSON);
        if(is_error($content)) {
            return error(-1, '调用接口获取微信公众号 jsapi_ticket 失败, 错误信息: ' . $content['message']);
        }
        if(empty($result) || intval(($result['errcode'])) != 0 || $result['errmsg'] != 'ok') {
            return error(-1, '获取微信公众号 jsapi_ticket 结果错误, 错误信息: ' . $result['errmsg']);
        }
        $record = array();
        $record['ticket'] = $result['ticket'];
        $record['expire'] = TIMESTAMP + $result['expires_in'] - 200;
        $this->account['jsapi_ticket'] = $record;
        upd_sys_core_cache('jsticket' . $this->account['key'], $record);
        return $record['ticket'];
    }


    public function getJssdkConfig($url = ''){
        global $_W;
        $jsapiTicket = $this->getJsApiTicket();
        if(is_error($jsapiTicket)){
            $jsapiTicket = $jsapiTicket['message'];
        }
        $nonceStr = random(16);
        $timestamp = TIMESTAMP;
        $url = empty($url) ? $_W['siteurl'] : $url;
        $string1 = "jsapi_ticket={$jsapiTicket}&noncestr={$nonceStr}&timestamp={$timestamp}&url={$url}";
        $signature = sha1($string1);
        $config = array(
            "appId"		=> $this->account['key'],
            "nonceStr"	=> $nonceStr,
            "timestamp" => "$timestamp",
            "signature" => $signature,
        );
        if(DEVELOPMENT) {
            $config['url'] = $url;
            $config['string1'] = $string1;
            $config['name'] = $this->account['name'];
        }
        return $config;
    }

    public function createModuleProcessor($name)
    {
        global $_W;
        $classname = "{$name}ModuleProcessor";
        if (!class_exists($classname)) {
            $file = IA_ROOT . "addons/{$name}/processor.php";
            if (!is_file($file)) {
                $classname = "DefaultModuleProcessor";
                $file = IA_ROOT . "/framework/wxgzh/processor.php";
            }
            if (!is_file($file)) {
                trigger_error('ModuleProcessor Definition File Not Found ' . $file, E_USER_WARNING);
                return null;
            }
            require $file;
        }
        if (!class_exists($classname)) {
            trigger_error('ModuleProcessor Definition Class Not Found', E_USER_WARNING);
            return null;
        }
        $o = new $classname();
        if ($o instanceof WeModuleProcessor) {
            return $o;
        } else {
            trigger_error('ModuleReceiver Class Definition Error', E_USER_WARNING);
            return null;
        }
    }
}

abstract class WeModuleProcessor
{
    public $message;
    public $uniacid;
    public $default_keyword = array();

    public function __construct()
    {
        global $_W;
        $this->uniacid = $_W['account']['uniacid'];
    }

    abstract function respond();

    protected function respSuccess()
    {
        return 'success';
    }

    protected function respText($content)
    {
        if (empty($content)) {
            return error(-1, 'Invaild value');
        }
        $content = str_replace("\r\n", "\n", $content);
        $response = array();
        $response['FromUserName'] = $this->message['to'];
        $response['ToUserName'] = $this->message['from'];
        $response['MsgType'] = 'text';
        $response['Content'] = htmlspecialchars_decode($content);
        preg_match_all('/\[U\+(\\w{4,})\]/i', $response['Content'], $matchArray);
        if (!empty($matchArray[1])) {
            foreach ($matchArray[1] as $emojiUSB) {
                $response['Content'] = str_ireplace("[U+{$emojiUSB}]", utf8_bytes(hexdec($emojiUSB)), $response['Content']);
            }
        }
        return $response;
    }

    protected function respImage($mid)
    {
        if (empty($mid)) {
            return error(-1, 'Invaild value');
        }
        $response = array();
        $response['FromUserName'] = $this->message['to'];
        $response['ToUserName'] = $this->message['from'];
        $response['MsgType'] = 'image';
        $response['Image']['MediaId'] = $mid;
        return $response;
    }

    protected function respVoice($mid)
    {
        if (empty($mid)) {
            return error(-1, 'Invaild value');
        }
        $response = array();
        $response['FromUserName'] = $this->message['to'];
        $response['ToUserName'] = $this->message['from'];
        $response['MsgType'] = 'voice';
        $response['Voice']['MediaId'] = $mid;
        return $response;
    }

    protected function respVideo(array $video)
    {
        if (empty($video)) {
            return error(-1, 'Invaild value');
        }
        $response = array();
        $response['FromUserName'] = $this->message['to'];
        $response['ToUserName'] = $this->message['from'];
        $response['MsgType'] = 'video';
        $response['Video']['MediaId'] = $video['MediaId'];
        $response['Video']['Title'] = $video['Title'];
        $response['Video']['Description'] = $video['Description'];
        return $response;
    }

    protected function respMusic(array $music)
    {
        if (empty($music)) {
            return error(-1, 'Invaild value');
        }
        global $_W;
        $music = array_change_key_case($music);
        $response = array();
        $response['FromUserName'] = $this->message['to'];
        $response['ToUserName'] = $this->message['from'];
        $response['MsgType'] = 'music';
        $response['Music'] = array(
            'Title' => $music['title'],
            'Description' => $music['description'],
            'MusicUrl' => $music['musicurl']
        );
        if (empty($music['hqmusicurl'])) {
            $response['Music']['HQMusicUrl'] = $response['Music']['MusicUrl'];
        } else {
            $response['Music']['HQMusicUrl'] = $music['hqmusicurl'];
        }
        if ($music['thumb']) {
            $response['Music']['ThumbMediaId'] = $music['thumb'];
        }
        return $response;
    }

    protected function respNews(array $news)
    {
        if (empty($news) || count($news) > 10) {
            return error(-1, 'Invaild value');
        }
        $news = array_change_key_case($news);
        if (!empty($news['title'])) {
            $news = array($news);
        }
        $response = array();
        $response['FromUserName'] = $this->message['to'];
        $response['ToUserName'] = $this->message['from'];
        $response['MsgType'] = 'news';
        $response['ArticleCount'] = count($news);
        $response['Articles'] = array();
        foreach ($news as $row) {
            $row = array_change_key_case($row);
            $response['Articles'][] = array(
                'Title' => $row['title'],
                'Description' => ($response['ArticleCount'] > 1) ? '' : $row['description'],
                'PicUrl' => $row['picurl'],
                'Url' => $row['url'],
                'TagName' => 'item'
            );
        }
        return $response;
    }

    protected function respCustom(array $message = array())
    {
        $response = array();
        $response['FromUserName'] = $this->message['to'];
        $response['ToUserName'] = $this->message['from'];
        $response['MsgType'] = 'transfer_customer_service';
        if (!empty($message['TransInfo']['KfAccount'])) {
            $response['TransInfo']['KfAccount'] = $message['TransInfo']['KfAccount'];
        }
        return $response;
    }
}
